/*  fgwirc - fungw based minimal irc client
    Copyright (C) 2019,2021  Tibor 'Igor2' Palinkas

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Project page and contact: http://repo.hu/projects/fgwirc
*/

#ifndef ACTION_H
#define ACTION_H

#include <libfungw/fungw.h>
#include "global_typedefs.h"

extern fgw_ctx_t fgw;

#define ACTION_MAX_ARGC 32

typedef enum {
	ACTION_HOOK_GO_ON, /* normal fgwirc processing is performed after the script finished */
	ACTION_HOOK_BLOCK  /* normal fgwirc processing is blocked by the script */
} action_hook_res_t;

typedef struct {
	const char *signature;
	int has_start;
	irc_server_t *server;
	const irc_from_t *from;
	int argc;
	char **argv;
	action_hook_res_t res;
} action_details_t;

extern const char actd_signature[];


void action_strarr(action_details_t *details, int argc, char **argv);
void action_str(const char *s);
void action_load_file(const char *fn);

void action_init(void);
void action_uninit(void);

void act_error(const char *actname, const char *msg);

#define ACT_ERROR(actname, msg) act_error(actname, "*** Error: " msg)

/* Require argument idx to exist and convert it to type; on success, also execute stmt */
#define ACT_CONVARG(idx, type, stmt) \
do { \
	if (argc <= idx) { \
		ACT_ERROR(argv[0].val.func->name, "not enough arguments"); \
		return FGW_ERR_ARGC; \
	} \
	if (fgw_arg_conv(&fgw, &argv[idx], type) != 0) { \
		ACT_ERROR(argv[0].val.func->name, "failed to convert argument " #idx " to " #type); \
		return FGW_ERR_ARG_CONV; \
	} \
	{ stmt; } \
} while(0)

/* If argument idx exists, convert it to type; on success, also execute stmt */
#define ACT_MAY_CONVARG(idx, type, stmt) \
do { \
	if (argc > idx) { \
		if (fgw_arg_conv(&fgw, &argv[idx], type) != 0) { \
			ACT_ERROR(argv[0].val.func->name, "failed to convert argument " #idx " to " #type); \
			return FGW_ERR_ARG_CONV; \
		} \
		{ stmt; } \
	} \
} while(0)

#define ACT_GDS_REST_(dst, idx, force) \
do { \
	int __n__; \
	if (argc <= idx) { \
		if (!force) break; \
		ACT_ERROR(argv[0].val.func->name, "not enough arguments"); \
		return FGW_ERR_ARGC; \
	} \
	for(__n__ = idx; __n__ < argc; __n__++) { \
		if (fgw_arg_conv(&fgw, &argv[__n__], FGW_STR) != 0) { \
			ACT_ERROR(argv[0].val.func->name, "ACT_TEXT_RES: failed to convert argument to FGW_STR"); \
			return FGW_ERR_ARG_CONV; \
		} \
		if (__n__ > idx) \
			gds_append((dst), ' ');\
		gds_append_str((dst), argv[__n__].val.str); \
	} \
} while(0)

#define ACT_GDS_REST(dst, idx) ACT_GDS_REST_(dst, idx, 1)
#define ACT_MAY_GDS_REST(dst, idx) ACT_GDS_REST_(dst, idx, 0)

#define ACT_SET_SERVER_AFFINITY(win, str, out_sid) \
do { \
	if (win->target.type != TARGET_NONE) { \
		act_error("server affinity", "*** Error: window is already in use"); \
		return FGW_ERR_ARG_CONV; \
	} \
	if (isdigit(*str)) { \
		char *__sep__ = strchr(str, '/'); \
		if (__sep__ == NULL) { \
			act_error("server affinity", "*** Error: Invalid address, should be 0/name"); \
			return FGW_ERR_ARG_CONV; \
		} \
		out_sid = atoi(str); \
		str = __sep__+1; \
	} \
	else \
		out_sid = win->target.sid; \
	if (out_sid < 0) { \
		act_error("server affinity", "*** Error: Invalid/unknown window server affinity"); \
		return FGW_ERR_ARG_CONV; \
	} \
} while(0)

/* Helper for getting the action details from ucc */
#define GET_ACTD(actd) \
	do { \
		actd = argv[0].val.argv0.user_call_ctx; \
		if (actd->signature != actd_signature) { \
			ACT_ERROR(argv[0].val.argv0.func->name, "missing action details"); \
			return FGW_ERR_ARG_CONV; \
		} \
	} while(0)

extern const irc_from_t *action_null_from;
#define action_details_reset(actd) \
do { \
	(actd)->has_start = 0; \
	(actd)->server = NULL; \
	(actd)->from = action_null_from; \
	(actd)->argc = 0; \
	(actd)->argv = NULL; \
	(actd)->res = ACTION_HOOK_GO_ON; \
} while(0);

#define ACTION_HELP_STR "\002help"
void action_help_print(const char *str, ...);
#define ACTION_HELP(help_text) \
do { \
	if ((argc > 1) && ((argv[1].type & FGW_STR) == FGW_STR) && (argv[1].val.str[0] == ACTION_HELP_STR[0]) && (strcmp(argv[1].val.str, ACTION_HELP_STR) == 0)) {\
		action_help_print help_text; \
		return 0; \
	} \
} while(0)

#endif
