/*  fgwirc - fungw based minimal irc client
    Copyright (C) 2019  Tibor 'Igor2' Palinkas

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Project page and contact: http://repo.hu/projects/fgwirc
*/

#include <stdio.h>
#include <termios.h>
#include <unistd.h>
#include <libporty/net/network.h>
#include <sockev/sockev.h>
#include <sockev/sockev_astdio.h>
#include <sockev/sockev_tcp4.h>
#include "window.h"
#include "input.h"
#include "action.h"
#include "edit.h"
#include "script.h"
#include "window_act.h"
#include "fgwirc_act.h"
#include "script_act.h"
#include "irc_act.h"
#include "irc.h"
#include "conf.h"
#include "ctcp.h"

time_t now;
struct tm now_tm;
win_t *wstatus;
int fgwirc_exit = 0;
int fgwirc_exit_val = 0;
int status_dirty;

static const char *user_search[] = {"../src_3rd/tcurses/tcurses_termcap", NULL};

void fgwirc_enter_str(const char *str)
{
	if (str[0] == '/') { /* action */
		action_str(str+1);
	}
	else { /* window message */
		win_t *curr = idx2win(win_curr_idx);
		if ((str[0] == '\\') && (str[1] == '/')) /* escape for leading / */
			str++;
		if (curr != NULL) {
			switch(curr->target.type) {
				case TARGET_CHANNEL:
				case TARGET_NICK:
					irc_send_win_msg(curr, str);
					break;
				case TARGET_SERVER_STATUS:
					irc_send_win_raw(curr, str);
					break;
				default:;
					/* notepad mode */
					win_append(curr, str);
			}
		}
	}
}

static void tick()
{
	static int logf_idx;
	static int last_hour = -1, last_minute = -1, last_second = -1;
	int n;
	win_t *curr = idx2win(win_curr_idx);

	if ((last_minute != now_tm.tm_min) || (last_hour != now_tm.tm_hour))
		status_dirty = 1;

#warning TODO: use binary action API

	if (last_second != now_tm.tm_sec) {
		action_str("on_tick_second");
		last_second = now_tm.tm_sec;
	}

	if (last_minute != now_tm.tm_min) {
		action_str("on_tick_minute");
		last_minute = now_tm.tm_min;
		last_hour = now_tm.tm_hour;
	}

	/* redraw the current window if anything happened */
	if ((curr != NULL) && (curr->dirty)) {
		win_draw(curr);
		status_dirty = 0;
	}

	/* redraw the status bar if needed */
	if (status_dirty) {
		win_draw_status(curr);
		win_flush();
		status_dirty = 0;
	}

	/* round robin log flusher: flush at most 1 log each tick */
	for(n = 0; n < windows.used; n++) {
		win_t *w = windows.array[n];

		logf_idx++;
		if (logf_idx >= windows.used)
			logf_idx = 0;

		if (w->needs_log_flush) {
			if (w->logf != NULL)
				fflush(w->logf);
			w->needs_log_flush = 0;
			break;
		}
	}
}

static void fgwirc_conf_init()
{
	conf_reg("fgwirc_exit_val", FGW_INT, &fgwirc_exit_val, NULL, "exit value of the fgwirc process once the exit or quit action is called");
}

static void set_now(void)
{
	now = time(NULL);
	now_tm = *localtime(&now);
}

static void load_rcfile(const char *rcfile)
{
	char tmp[1024], msg[1100];

	if (*rcfile == '~') {
		const char *home = getenv("HOME");

		if (home == NULL) {
			fprintf(stderr, "Error: can't resolve ~ because there's no $HOME (rc file load)\n");
			exit(1);
		}
		if (strlen(home) + strlen(rcfile) > sizeof(tmp) - 4) {
			fprintf(stderr, "Error: rc file path too long\n");
			exit(1);
		}

		sprintf(tmp, "%s%s", home, rcfile+1);
		rcfile = tmp;
	}

	sprintf(msg, "Loading rc file '%s'", rcfile);
	win_append(windows.array[0], msg);
	action_load_file(rcfile);
}

static int help(const char *prog, const char *topic)
{
	printf("fgwirc - fungw based interactive IRC client\n");
	printf("Invocation: %s [-c rcfile]\n", prog);
	printf("\n");
	return 0;
}

int main(int argc, char *argv[])
{
	const char *rcfile = "~/.fgwirc.rc";
	int n;

	tcurses_cap_search_user = user_search;

	conf_init();

	for(n = 1; n < argc; n++) {
		char *cmd= argv[n], *arg = argv[n+1];
		while(*cmd == '-') cmd++;
		switch(*cmd) {
			case 'c': rcfile = arg; n++; break;
			case 'h': return help(argv[0], arg);
			default: fprintf(stderr, "Unknown command line argument: '%s'\n", argv[n]); return 1;
		}
	}

	P_net_init();
	win_init();
	input_init();

	action_init();
	edit_init();
	script_init();
	window_act_init();
	fgwirc_act_init();
	script_act_init();
	ctcp_init();
	irc_act_init();
	irc_init();
	fgwirc_conf_init();

	set_now();
	wstatus = win_new("{fgwirc}");
	win_set_target(wstatus, TARGET_FGWIRC_STATUS, -1, "{fgwirc}", NULL);
	win_show(wstatus);

	script_clean_hooks();
	load_rcfile(rcfile);
	tick();
	for(; fgwirc_exit == 0;) {
		sockev_poll(1000);
		set_now();
		tick();
	}

	/* flush pending quit messages */
	for(n = 0; n < 200; n++)
		sockev_poll(1);

	script_uninit();
	action_uninit();
	conf_uninit();
	return fgwirc_exit_val;
}
